(function($) {


  window.State = {
    section: null,
    group: null,
    model: null,
    preview: 0,
    docs: 0,
    notes: 0,
    index: false,
    code: false,
    modelStr: false
  };

  getParameters();
  setIndex();

  if (!State.docs && !State.notes) {

      var modelSet = getDataSet();

    if (State.bundle) {
      modelSet = removeDuplicates(modelSet.concat(getModelBundle(getModelString())));
    }

    if (State.preview) {
      $('html').addClass('preview');
      renderModels(modelSet);
    } else {
      renderModels(modelSet);
    }

    $('.sm-model-frame').attr('scrolling', 'yes');

  } else if (State.docs) {

    $('.sm-toolbar-container').addClass('isVisible');
    $('.sm-documentation-container').addClass('isActive');
    renderDocumentation(window.Docs);

  } else if (State.notes) {

    $('.sm-toolbar-container').addClass('isVisible');
    $('.sm-documentation-container').addClass('isActive');
    renderDocumentation(window.Notes);

  }

  







  // PARSE URL PARAMETERS

  var remoteKeypressScript = '<script type="text/javascript">document.addEventListener("keyup", function(e) { window.parentIFrame.sendMessage({type: "keyup", code: e.keyCode}); }); document.addEventListener("keydown", function(e) { window.parentIFrame.sendMessage({type: "keydown", code: e.keyCode}); });</script>';
  

  if (window.Notes) {
    $('.sm-notes-button').addClass('isAvailable');
  }


  function buildDocumentationNav(data) {
    
    var output = data.splash.link;
    
    output += '<ul class="sm-docs-nav">';

    for (var section in data.structure) {
      output += data.structure[section].link;
      if (data.structure[section].subheadings) {
        output += data.structure[section].subheadings;
      }
      output += '</li>';
    }

    return output;

  }

  function renderDocumentation(data) {

    $('.sm-documentation-nav').html(buildDocumentationNav(data));
    $('.sm-documentation-content').append(data.splash.content);

    for (var section in data.structure) {
      $('.sm-documentation-content').append(data.structure[section].content);
    }

    $('.sm-documentation-content h3').each(function() {
      $(this).prepend('<div id="'+$(this).attr('id')+'" style="position: relative; top: -80px;"></div>');
      $(this).attr('id', '');
    })

  }


  function toArray(thing) {
    if (Array.isArray(thing)) return thing;
    else {
      var arr = []; arr.push(thing);
      return arr;
    }
  }


  function find(content, regs, arr, num) {
    var matches = [], match, arrayOut = arr, captured = num ? num : 1, i, m;

    if (arrayOut === undefined) arrayOut = true;

    toArray(regs).forEach(function(rgex) {
      while ((match = (rgex).exec(content)) !== null) {
        if (captured === 1) matches.push(match[1])
        else {
          m = {};
          for (i = 1; i <= captured; i++ ) m[i] = match[i];
          matches.push(m);
        }
      }
    })

    return arrayOut ? matches : matches[0];
  }


  function resetState() {
    State.section = null;
    State.group = null;
    State.model = null;
    State.preview = 0;
    State.bundle = 0;
    State.docs = 0;
    State.notes = 0;
  }

  function removeDuplicates(list) {
    return list.filter(function(item, i) {
      return list.indexOf(item) === i;
    })
  }

  function updateStateView(str) {
    resetState();
    str = str
      .split('|')
      .forEach(function(part, i) {
        if (i === 0) State.section = decodeURIComponent(part);
        if (i === 1) State.group = decodeURIComponent(part);
        if (i === 2) State.model = decodeURIComponent(part);
      });
  }

  function getParameters() {

    var params = window.location.search;
    var models = params.match(/\?models=(.+?)&/);
    var preview = params.match(/\&preview=(\d)/);
    var bundle = params.match(/\&bundle=(\d)/);
    var docs = params.match(/docs=(\d)/);
    var notes = params.match(/notes=(\d)/);

    if (models) updateStateView(models[1]);
    if (preview) State.preview = preview[1] * 1;
    if (bundle) State.bundle = bundle[1] * 1;
    if (docs) State.docs = docs[1] * 1;
    if (notes) State.notes = notes[1] * 1;

  }

  function setIndex() {
    var section, group, model, data = window.Data;

    section = $('[rel="'+State.section+'"]');

    if (State.group) {
      section.children().addClass('isActive');
      group = section.find('[rel="'+State.group+'"]');

      if (State.model) {
        group.children().addClass('isActive');
        model = group.find('[rel="'+State.model+'"]');
        model.children().addClass('isActive');
      } else {
        group.children().first().find('.sm-index-alltext').addClass('isActive');
      }

    } else {
      section.children().first().find('.sm-index-alltext').addClass('isActive');
    }

    if (State.docs) $('.sm-docs-button').addClass('isActive');
    if (State.notes) $('.sm-notes-button').addClass('isActive');

  }

  function getDataSet() {

    var set = [],
        data = window.Data;

    if (State.model) {
      if (!data[State.section][State.group][State.model].hidden) {
        set.push(data[State.section][State.group][State.model]);
      }
    } else {
      if (State.group) {
        for (var model in data[State.section][State.group]) {
          if (!data[State.section][State.group][model].hidden) {
            set.push(data[State.section][State.group][model])
          }
        }
      } else {
        if (State.section) {
          for (var group in data[State.section]) {
            for (var model in data[State.section][group]) {
              if (!data[State.section][group][model].hidden) {
                set.push(data[State.section][group][model])
              }
            }
          }
        }
      }
    }
    return set;
  }

  function getModelRef(str) {
    var match = str.match(/\|([^\|]+)$/);
    if (match) {
      return match[1];
    } else {
      console.error('No model reference in this string');
    }
  }


  function getModelString() {
    return ([State.section, State.group, State.model])
      .filter(function(n) { return !!n; }).join('|');
  }


  function getModelObj(str) {
    var list = str.split('|');
    return window.Data[list[0]][list[1]][list[2]];
  }


  function getModelBundle(str) {
    var list = getModelObj(str).references;
    if (list.length > 0) {
      return list.map(function(ref) {
        return getModelObj(ref);
      }).reverse();
    } else {
      return empty = [];
    }
  }


  function renderModels(set) {

    $('.sm-toolbar-container').addClass('isVisible');
    $('.sm-stage-container').addClass('isActive');

    var stage = $('.sm-stage-container');
    var output = '';
    var px = stage.width();
    var em = Math.round((stage.width()/16) * 100) / 100;

    set.forEach(function(model) {
      
      if (!model.scaffold) {

        var id = model.section + '|' + model.group + '|' + model.display; 

        output += '<div class="sm-frame-container" rel="'+id+'">\n';

        output += '  <div class="sm-info-container">\n';
        output += '    <div class="sm-info-extra">\n';
        output += '      <div class="sm-extra-icon fa fa-ellipsis-h"></div>\n';
        output += '      <div class="sm-extra-modal">\n';
        output += '        <div class="sm-extra-buttons" data-model="'+id+'">\n';
        output += '          <button class="sm-code-btn"><i class="fa fa-code"></i>Code</button>\n';
        output += '          <button class="sm-preview-btn"><i class="fa fa-eye"></i>Preview</button>\n';
        if (model.references.length > 0) {
          output += '          <button class="sm-bundle-btn"><i class="fa fa-cubes"></i>Bundle</button>\n';
        } else {
          output += '<div style="display:inline-block;width:5em"></div>'
        }
        output += '        </div>\n';

        if (model.note) {
        output += '        <div class="sm-extra-note">'+model.note+'</div>\n';
        }

        output += '        <ul class="sm-extra-list">\n';
        output += '          <li><span class="sm-extra-item">Name:</span>';
        output += '              <span class="sm-extra-value">' + model.name + '</span></li>\n';
        output += '          <li><span class="sm-extra-item">File:</span>';
        output += '              <span class="sm-extra-value">' + model.file + '</span></li>\n';
        if (model.author) {
          output += '          <li><span class="sm-extra-item">Author:</span>';
          output += '              <span class="sm-extra-value">' + model.author + '</span></li>\n';
        }
        if (model.global) {
          output += '          <li><span class="sm-extra-item">Global:</span>';
          output += '              <span class="sm-extra-value">Yes</span></li>\n';
        }
        if (model.js.length > 0) {
          output += '          <li><span class="sm-extra-item">JS:</span>';
          output += '              <span class="sm-extra-value">' + model.js.join(', ') + '</span></li>\n';
        }
        if (model.references.length > 0) {
          output += '          <li><span class="sm-extra-item">Calls:</span>';
          output += '              <span class="sm-extra-value">';
          output += removeDuplicates(model.references.map(function(ref) {
            if (getModelObj(ref).scaffold) {
              return getModelRef(ref);
            } else {
              return '<span class="sm-extra-link" data-url="'+ref+'">'+getModelRef(ref)+'</span>';
            }
          })).reverse().join(', ');
          output += '</span></li>\n'
        }
        if (model.includes.length > 0) {
          output += '          <li><span class="sm-extra-item">Includes:</span>';
          output += '<span class="sm-extra-value">';
          output += removeDuplicates(model.includes.map(function(ref) {
            if (getModelObj(ref).scaffold) {
              return getModelRef(ref);
            } else {
              return '<span class="sm-extra-link" data-url="'+ref+'">'+getModelRef(ref)+'</span>';
            }
          })).reverse().join(', ');
          output += '</span></li>\n'
        }

        output += '        </ul>\n';
        output += '      </div>\n';
        output += '    </div>\n';
        output += '    <div class="sm-info-name">' + model.display;
        if (model.wip) {
          output += '    <span class="sm-info-wip">';
          output += '      <i class="fa fa-exclamation-circle sm-wip-icon"></i>WIP';
          output += '    </span>';
        }
        output += '    </div>\n';
        output += '  </div>\n';

        output += '  <div class="sm-frame-width">\n';
        output += '    <input class="sm-width-px" type="text" value="'+px+'" />px / \n';
        output += '    <input class="sm-width-em" type="text" value="'+em+'" />em\n';
        output += '  </div>';
        output += '  <iframe class="sm-model-frame" scrolling="yes" src="models/'+model.name+'.html"></iframe>\n';
        output += '  <div class="sm-frame-slider"></div><div class="sm-frame-overlay"></div>';

        output += '</div>';

      }

    });

    stage.append(output);

  }


  function renderPreview(set) {

    $('html').addClass('preview');

    var preview = $('.sm-preview-container');
    var model = set[0];
    var frame = '<iframe class="sm-model-preview" scrolling="no" src="models/'+model.name+'.html"></iframe>';

    preview.html(frame);

  }
  

  // UPDATING VIEW

  function updateURL(reset, trump) {

    var reset = reset || false, 
        url = '', 
        href;

    if (!reset) {
      url += '?models=';
      for (var key in State) {
        if (key === 'section' && State[key]) {
          url += encodeURIComponent(State[key]);
        } else if(key === 'group' && State[key]) {
          url += '|' + encodeURIComponent(State[key]);
        } else if(key === 'model' && State[key]) {
          url += '|' + encodeURIComponent(State[key]);
        }
      }
      url += '&preview=' + State.preview;
      url += '&bundle=' + State.bundle;
    }

    if(trump) {
      url = trump;
    }
    
    if (window.location.search) {
      href = window.location.href.replace(window.location.search, url);
    } else {
      href= window.location.href + url;
    }

    window.location.href = href.replace(/(#.*)$/,'');

  }


  function openModelIndex() {
    $('.sm-index-button').addClass('isActive');
    $('.sm-index-container').addClass('isActive');
    State.index = true;
  }

  function closeModelIndex() {
    $('.sm-index-button').removeClass('isActive');
    $('.sm-index-container').removeClass('isActive');
    State.index = false;
  }


  inputs = {

    '.sm-dropbtn-section, .sm-dropbtn-group': {
      events: {
        'click': function(it) {
          var btn = $(it.target);
          if (btn.hasClass('sm-index-itemtext')) {
            btn = btn.parent();
          }
          if (btn.hasClass('sm-index-alltext')) return;
          if (btn.hasClass('isActive')) {
            btn.removeClass('isActive').next().removeClass('isActive');
          } else {
            btn.addClass('isActive').next().addClass('isActive');
          }
        }
      }
    },

    '.sm-index-itembtn, .sm-index-alltext': {
      events: {
        'click': function(it) {

          var btn = $(it.target);

          var itemIcon = btn.closest('.sm-index-itemicons').length > 0;
          var isScaffold = btn.hasClass('scaffold') || btn.closest('.scaffold').length > 0;
          

          if (itemIcon || isScaffold) {
            return;
          }

          if (btn.hasClass('sm-index-itemtext')) {
            btn = btn.parent();
          }

          if (btn.hasClass('isActive')) {

            return;

          } else {

            $('.sm-index-itembtn.isActive, .sm-index-alltext.isActive').removeClass('isActive');

            btn.addClass('isActive');

            if (btn.hasClass('sm-index-alltext')) {
              setTimeout(function() {
                inputs['.sm-index-button'].value = false;
                inputs['.sm-index-button'].onFalse();
              }, 200);
            }

            updateStateView(btn.attr('data-url'));
            updateURL();
          }

        }
      }
    },

    '.sm-index-button': {
      value: false,
      onTrue: function() {
        openModelIndex();
      },
      onFalse: function() {
        closeModelIndex();
      },
      events: {
        'click': function() {
          inputs['.sm-index-button'].value = !inputs['.sm-index-button'].value;
          updateView();
        }
      }
    },

    '.sm-docs-button': {
      value: false,
      onTrue: function() {
        $('.sm-docs-button').addClass('isActive');
      },
      onFalse: function() {
        $('.sm-docs-button').removeClass('isActive');
      },
      events: {
        'click': function() {
          inputs['.sm-index-button'].value = false;
          inputs['.sm-notes-button'].value = false;
          inputs['.sm-docs-button'].value = !inputs['.sm-docs-button'].value;
          resetState();
          updateURL(null, '?docs=1');
        }
      }
    },

    '.sm-notes-button': {
      value: false,
      onTrue: function() {
        $('.sm-notes-button').addClass('isActive');
      },
      onFalse: function() {
        $('.sm-notes-button').removeClass('isActive');
      },
      events: {
        'click': function() {
          inputs['.sm-index-button'].value = false;
          inputs['.sm-docs-button'].value = false;
          inputs['.sm-notes-button'].value = !inputs['.sm-notes-button'].value;
          resetState();
          updateURL(null, '?notes=1');
        }
      }
    }

  }


  if (State.docs) inputs['.sm-docs-button'].value = true;
  if (State.notes) inputs['.sm-notes-button'].value = true;


  function updateView() {
    for (var input in inputs) {
      if (inputs[input].hasOwnProperty('value')) {
        if (inputs[input].value) {
          inputs[input].onTrue();
        } else {
          inputs[input].onFalse();
        }
      }
    }
  }

  for (var input in inputs) {
    for (var event in inputs[input].events) {
      $(input).on(event, inputs[input].events[event]);
    }
  }




  // IFRAME RESIZING

  $(window).on('load', function() {

    /**
     *  Display rendered models with fade in animation
     */
    $('.sm-frame-container').each(function(i, m) {
      var model = $(m);
      setTimeout(function() {
        model.addClass('isVisible');
      }, i * 100);
    })

    function exitPreview() {
      State.preview = 0;
      updateURL();
    }

    window.FRAMES = iFrameResize({
      log: false,
      heightCalculationMethod: 'lowestElement',
      resizeFrom: 'child',
      messageCallback: function(received) {
        processKeyboardShortcut(received.message.code, received.message.type);
      }
    })

    // Change scrolling to yes
    $('.sm-model-frame').attr('scrolling', 'yes');
    $('.sm-model-preview').attr('scrolling', 'yes');


    var slider = $('.sm-frame-slider'),
        codeSlider = $('.sm-code-slider'),
        fadeBlocks = $('.sm-fade-top, .sm-fade-bottom'),
        resizing = false,
        startX,
        startWidth,
        container,
        overlay = null,
        offset,
        newWidth,
        modelSlide = false,
        codeSlide = false;


    slider.on('mousedown', function(e) {
      modelSlide = true;
      container = $(this).parent();
      overlay = $(this).next();
      resizing = true;
      startX = (e.clientX);
      startWidth = (container.width());
    })

    codeSlider.on('mousedown', function(e) {
      codeSlide = true;
      overlay = $('.sm-code-overlay, .sm-code-wrap-overlay');
      container = $(this).parent();
      resizing = true;
      startX = e.clientX;
      startWidth = container.width();
    })

    function resetWidth(container, width) {
      container.css('width', width);
      container.find('.sm-width-px').val(width);
      container.find('.sm-width-em').val(Math.round((width / 16) * 100) / 100);
    }


    $('*')
      .on('mousemove', function(e) {
        if (!resizing) return;
        if (overlay) overlay.addClass('active');
        offset = (startX - e.clientX);
        newWidth = (startWidth - (offset));
        container.css('width', newWidth);
        if (modelSlide) {
          container.find('.sm-width-px').val(newWidth);
          container.find('.sm-width-em').val(Math.round((newWidth / 16) * 100) / 100);
        }
        if (codeSlide) {
          if (newWidth < 555 && !container.hasClass('lt555')) {
            container.addClass('lt555');
          }
          if (newWidth > 555 && container.hasClass('lt555')) {
            container.removeClass('lt555');
          }
          fadeBlocks.css('width', newWidth);
        }
      })
      .on('mouseup', function(e) {
        if (resizing) {
          resizing = false;
          modelSlide = false;
          codeSlide = false;
          if (overlay) {
            overlay.removeClass('active');
            overlay = null;
          }
        }
      })



    $(window).on('resize', function() {

      $('.sm-frame-container').each(function() {

        if ( ($(this).width() + $(this).offset().left + 4) > $(window).width() ) {
          $(this).css('width', '100%');
        }

        $(this).find('.sm-width-px').val($(this).width());
        $(this).find('.sm-width-em').val(Math.round(($(this).width() / 16) * 100) / 100);

      })

      var codeContainerWidth = $('.sm-code-container').width();

      if (codeContainerWidth < 555 && !$('.sm-code-container').hasClass('lt555')) {
        $('.sm-code-container').addClass('lt555');
      }
      if (codeContainerWidth > 555 && $('.sm-code-container').hasClass('lt555')) {
        $('.sm-code-container').removeClass('lt555');
      }


    })


    /**
     *  Update iframe width when enter key is pressed in width input
     */
    var focusedInput;
    $('.sm-width-px, .sm-width-em').keyup(function(e) {
      if (e.keyCode === 13) {
        var w = '';
        var container = $(this).closest('.sm-frame-container');

        if ($(this).hasClass('sm-width-em')) {
          resetWidth(container, Math.round($(this).val() * 16));
        } else {
          resetWidth(container, $(this).val());
        }

        $(this).blur();
      }
    })

    
    /**
     *  Links in "Calls" info section generate new model views
     */
    $('.sm-extra-link').on('click', function() {
      updateStateView($(this).attr('data-url'));
      updateURL();
    })


    /**
     *  Resets code modal settings on each button click or exit
     */
    function closeSassdocModal() {
      $('.sm-sassdoc-wrap').removeClass('isActive');
      setTimeout(function() {
        $('.sm-sassdoc-contents').empty();
      }, 250);
    }

    $('.sm-sassdoc-exit').click(closeSassdocModal);

    function closeCodeModal() {
      $('.sm-style-dependencies-btn').unbind();
      $('.sm-toolbar-location').empty();
      $('.sm-code-container').removeClass('isActive');
      closeSassdocModal();
      $('.sm-sassdoc-link').unbind();
      setTimeout(resetCodeModal, 100);
    }

    function resetCodeModal(resetBtns) {
      var reset = resetBtns === undefined || false;
      if (resetBtns) {
        $('.sm-code-buttons').children().removeClass('isActive');
        $('.sm-sass-btn').addClass('isActive');
      }
      $('.sm-pre-tag').removeClass('language-css language-sass language-html');
      State.code = false;
    }

    function convertSassdocContentLinks() {
      $('.sm-sassdoc-contents .item__cross-type ~ a').each(function() {
        $(this).click(function(e) {
          e.preventDefault();
          var href = $(this).attr('href');
          var matches = find(href, /(.+-(variable|placeholder|function|mixin)-(.+))/g, true, 3);
          var category = matches[0]['2'] + 's';
          var name = matches[0]['3'];
          initSassdocModal(category, name, false)
        });
      })
    }

    function initSassdocModal(category, name, makeActive) {
      makeActive = makeActive || true;
      $('.sm-sassdoc-contents').html(window.Sassdocs[category][name]);
      $('.sm-sassdoc-contents .item__heading').each(function() {
        $(this).html($(this).children().first().text());
        if (category === 'placeholders') $(this).prepend('%');
        if (category === 'variables') $(this).prepend('$');
      })
      convertSassdocContentLinks();
      activateSassdocLinks('.sm-sassdoc-contents .item__code-wrapper pre');
      activateSassdocLinks('.sm-sassdoc-contents .item__example pre');
      if (makeActive) {
        $('.sm-sassdoc-wrap').addClass('isActive');
      }
    }

    function setSassdocLinkHandler(text, obj, category) {
      $(obj).html('<span class="sm-sassdoc-link">' + $(obj).text() + '</span>');
      $(obj).click(function() {
        initSassdocModal(category, text);
      })
    }

    function convertToSassdocLink(preTag, category, className) {
      $(preTag + ' .token.' + className).each(function() {

        var group = category;
        var text = $(this).text();

        if (text === $('.sm-sassdoc-contents .item__heading').text()) return;

        if (className === 'placeholder' || className === 'variable') {
          text = text.replace(/\$|%/,'');
        }

        if ($(this).prev().text() === '@include') {
          group = 'mixins';
        }

        if (window.Sassdocs[group].hasOwnProperty(text)) {
          setSassdocLinkHandler(text, this, group);
        }

      })
    }

    function activateSassdocLinks(preTag) {
      
      convertToSassdocLink(preTag, 'variables', 'variable');
      convertToSassdocLink(preTag, 'placeholders', 'placeholder');
      convertToSassdocLink(preTag, 'functions', 'function');

      // Handles mixins that are called without parameters
      $(preTag + ' .token.keyword').each(function() {
        if ($(this).text() === '@include') {
          var next = $(this)[0].nextSibling;
          var text = next.nodeValue.trim();
          if (text) {
            $(this).append(' ');
            var link = $('<span class="sm-sassdoc-link">' + text + '</span>');
            $(next).replaceWith(link);
            $(link).click(function() {
              initSassdocModal('mixins', text);
            })
          }
        }
      });

    }
    
    function openCodeModal(modelStr, language) {

      State.code = true;

      var modelDisplay = $('.sm-toolbar-location');
      var preTag = $('.sm-pre-tag');
      var container = $('.sm-code-container');
      var lang = language || 'Sass';

      if (lang === 'Sass') {
        preTag.addClass('language-sass')
        .html(getModelObj(modelStr).highlightedSass);
        activateSassdocLinks('.sm-pre-tag');
      } else if (lang === 'CSS') {
        preTag.addClass('language-css')
        .html(getModelObj(modelStr).highlightedCss);
      } else if (lang === 'HTML') {
        preTag.addClass('language-html')
        .html(getModelObj(modelStr).highlightedInterpretation);
      } else if (lang === 'Template') {
        preTag.addClass('language-html')
        .html(getModelObj(modelStr).highlightedTemplate);
      }

      modelDisplay.html(modelStr.replace(/\|/g, ' | '));

      container
        .find('.sm-code-buttons')
        .attr('data-model', modelStr);

      if (!container.hasClass('isActive')) {
        container.addClass('isActive');
      }

      $('.sm-style-dependencies-btn').click(styleDependencyHandler);
      $('.sm-style-dependent-btn').click(styleDependentHandler);

    }

    function styleDependencyHandler() {
      var btn = $(this);
      btn.parent().toggleClass('isActive');
      if (btn.parent().hasClass('isActive')) {
        btn.text('Hide Dependencies');
      } else {
        btn.text('Show Dependencies');
      }
    }

    function styleDependentHandler() {
      var btn = $(this);
      var active = btn.hasClass('isActive');
      var fill = $('.sm-style-dependent-fill');
      $('.sm-style-dependent-btn').removeClass('isActive');
      fill.html('');
      if (!active) {
        btn.addClass('isActive');
        var content = btn.next().clone(true, true);
        $('.sm-style-dependent-fill').append(content);
      }
    }

    function getActiveCodeLang() {
      return $('.sm-code-buttons button.isActive').text();
    }


    /**
     *  Opens code modal and inserts sass data as default view
     */
    $('.sm-code-btn').on('click', function() {
      var model = $(this).parent().attr('data-model');
      openCodeModal(model);
    })


    function resetScaffoldModal() {
      $('.sm-index-itembtn').removeClass('isActive');
      $('.scaffold.isActive').removeClass('isActive');
      $('.sm-index-scaffold.isActive').removeClass('isActive');
    }

    $('.sm-index-itembtn .fa').click(function() {

      var btn = $(this);
      var model = btn.parent().parent().attr('data-url');
      var code = btn.hasClass('fa-code');
      var preview = btn.hasClass('fa-eye');
      var bundle = btn.hasClass('fa-cubes');
      var info = btn.hasClass('fa-info-circle');

      model = decodeURIComponent(model);

      if (info) {

        var active = $(this).closest('.scaffold').hasClass('isActive');

        resetScaffoldModal();

        if (!active) {
          $(this).closest('.scaffold').addClass('isActive');
          $(this).next().addClass('isActive');
        } else {

        }
        
      } else if (code) {

        resetScaffoldModal();

        if ($('.sm-code-container').hasClass('isActive')) {

          var dataUrl = decodeURIComponent($(this).closest('[data-url]').attr('data-url'));
          var same = $('.sm-code-buttons').attr('data-model') === dataUrl;

          if (same) {
            closeCodeModal();
          } else {
            resetCodeModal(false);
            openCodeModal(model, getActiveCodeLang());
          }

        } else {

          openCodeModal(model);

        }

      } else if (preview) {

        enterPreview(model);

      } else if (bundle) {

        viewBundle(model);
        
      }

    })

    
    /**
     *  Exits code modal and resets settings
     */
    $('.sm-code-exit-btn').on('click', function() {
      closeCodeModal();
    })


    /**
     *  Handler to manage code tabs
     */
    var codeButtons = $('.sm-code-buttons').children();

    codeButtons.on('click', function() {

      $('.sm-style-dependencies-btn').unbind();

      codeButtons.removeClass('isActive');
      var type = $(this).text();
      var model = getModelObj($(this).parent().attr('data-model'));
      var pre = $('.sm-pre-tag');
      $(this).addClass('isActive');

      pre.removeClass('language-css language-sass language-html');

      if (type === 'CSS') {
        pre.addClass('language-css');
        pre.html(model.highlightedCss);
      } else if (type === 'Sass') {
        pre.addClass('language-sass');
        pre.html(model.highlightedSass);
        activateSassdocLinks('.sm-pre-tag');
      } else if (type === 'HTML') {
        pre.addClass('language-html');
        pre.html(model.highlightedInterpretation);
      } else if (type === 'Template') {
        pre.addClass('language-html');
        pre.html(model.highlightedTemplate);
      }

      $('.sm-style-dependencies-btn').click(styleDependencyHandler);
      $('.sm-style-dependent-btn').click(styleDependentHandler);

    })


    /**
     *  Handles Preview iFrame Resizing
     */
    function resizePreview() {
      var windowHeight = $('html').css('height').replace('px','');
      var frameStyle = $('.sm-model-preview').attr('style');
      var frameHeight = frameStyle.match(/height:\s*([\d\.]+)/)[1];
      var ltWindow = windowHeight * 1 > frameHeight * 1;
      if (ltWindow) {
        $('.sm-model-preview').addClass('ltWindow');
      } else {
        $('.sm-model-preview').removeClass('ltWindow');
      }
    }


    


    if (State.preview) {

      var notice = $('.sm-preview-notice');
      notice.addClass('isDisplayed');

      State.modelStr = getModelString();
      $('.sm-code-container, .sm-index-container').addClass('inPreview');

      // Display and remove notice
      setTimeout(function() {
        // resizePreview();
        notice.addClass('isEntering');
        setTimeout(function() {
          notice.removeClass('isEntering');
          setTimeout(function() {
            notice.removeClass('isDisplayed');
          }, 250);
        }, 2000);
      }, 300);
 
    }

    $('.sm-model-frame').each(function() {
      $(this).contents().find('body').append(remoteKeypressScript);
    });


    document.addEventListener('keyup', function(e) { 
      processKeyboardShortcut(e.keyCode, 'keyup'); 
    });

    document.addEventListener('keydown', function(e) {
      processKeyboardShortcut(e.keyCode, 'keydown');
    })


    function getFirstInStage() {
      var models = $('.sm-frame-container.isVisible');
      if (models.length > 0) {
        return models.attr('rel');
      } else {
        return null;
      }
    }


    /**
     *  Handles keyboard shortcuts
     *  alt + c = code modal (for first model in view)
     *  alt + p = preview modal (for first model in view)
     *  alt + m = model index modal
     *  alt + n = notes
     *  alt + d = documentation
     */
    var alt_pressed = false;

    function processKeyboardShortcut(keycode, type) {

      var inDocs = State.docs || State.notes,
          keyup = type === 'keyup',
          keydown = type === 'keydown',
          ALT = keycode === 18,
          C = keycode === 67,
          P = keycode === 80,
          M = keycode === 77,
          N = keycode === 78,
          H = keycode === 72;

      if (keyup && ALT) alt_pressed = false;
      if (keydown && ALT) alt_pressed = true;

      if (!inDocs && alt_pressed && keyup && C) {
        if (State.code) {
          closeCodeModal();
        } else {
          if (State.preview) {
            openCodeModal(State.modelStr);
          } else {
            openCodeModal(getFirstInStage());
          }
        }
      }
      if (!inDocs && alt_pressed && keyup && P) {
        if (State.preview) {
          console.log('hmm');
          exitPreview(keycode);
        } else {
          enterPreview(getFirstInStage());
        }
      }
      if (alt_pressed && keyup && M) {
        if (State.index) {
          closeModelIndex();
        } else {
          openModelIndex();
        }
      }
      if (alt_pressed && keyup && N) {
        State.notes = 1;
        updateURL(null, '?notes=1');
      }
      if (alt_pressed && keyup && H) {
        State.docs = 1;
        updateURL(null, '?docs=1');
      }


    }


    function enterPreview(modelStr) {
      updateStateView(modelStr);
      State.preview = 1;
      updateURL();
    }

    function viewBundle(modelStr) {
      updateStateView(modelStr);
      State.bundle = 1;
      updateURL();
    }


    /**
     *  Update State on bundle click
     */
    $('.sm-bundle-btn').on('click', function() {
      viewBundle($(this).parent().attr('data-model'));
    })

    /**
     *  Update State on preview click
     */
    $('.sm-preview-btn').on('click', function() {
      enterPreview($(this).parent().attr('data-model'));
    })
  })


  /**
   *
   *  Handles documentation navigation
   *
   */

  var winHash = window.location.hash;

  if ((State.docs || State.notes) && winHash !== '') {
    var hashObj = parseHash(winHash);
    $('.sm-docs-nav-section-text[href="#'+hashObj.section+'"]').each(function() {
      toggleSection(this);
    })
    toggleSubheading(hashObj);
  } else if (State.docs || State.notes) {
    setTimeout(function() {
      $('.sm-documentation-content').children().first().addClass('isVisible');
    }, 200);
    
  }

  
  function parseHash(hash) {
    var section = find(hash, /([^\+]+)/g, false);
    var subheading = find(hash, /(.+\+.+)/g, false);
    return {
      section: section.replace(/^#/,''),
      subheading: subheading
    }
  }

  function getNavSection(data) {
    data = '#' + data;
    data = data.replace(/##/, '#').replace(/(\+.+)$/g, '');
    return $('.sm-docs-nav-section-text[href="'+data+'"]');
  }

  function getNavSub(data) {
    data = '#' + data;
    data = data.replace(/##/, '#');
    return $('.sm-docs-nav-sublink[href="'+data+'"]');
  }

  function currSection() {
    return $('.sm-docs-section-wrap.isActive');
  }

  function scrollToTop() {
    window.scrollTo(0,0);
  }

  function toggleSubheading(hashdata) {
    if (hashdata.subheading) {
      var sublink = $('.sm-docs-nav-sublink[href="'+hashdata.subheading+'"]');
      var subheading = $('[id="'+hashdata.subheading.replace(/^#/,'')+'"]');
      sublink.addClass('isActive');
      setTimeout(function() {
        $(document).scrollTop(subheading.offset().top - 15);
      }, 300);
    }
  }

  function closeContentSection() {
    var currentSection = currSection();
    currentSection.removeClass('isActive isVisible');
    $('.sm-docs-nav-section-text[href="#'+currentSection.attr('data-content')+'"]').removeClass('isActive');
  }

  function openContentSection(hashdata) {
    var section = $('.sm-docs-section-wrap[data-content="'+hashdata.section+'"]');
    setTimeout(function() {
      section.addClass('isActive');
      setTimeout(function() {
        section.addClass('isVisible');
      }, 200);
    }, 20);
    if (hashdata.subheading) {
      $(section.subheading).addClass('isActive');
    } else {
      setTimeout(scrollToTop, 1);
    }

  }

  function toggleSection(section) {
    $('.sm-docs-nav-sublink').removeClass('isActive');
    $('.sm-docs-nav-section-text').each(function() {
      if (this === section) {
        if ($(this).hasClass('isActive')) {
          $(this).toggleClass('isOpen');
        } else {
          $(this).addClass('isActive isOpen');
          closeContentSection();
          openContentSection(parseHash($(this).attr('href')));
        }
      } else {
        $(this).removeClass('isActive').find('.isActive').removeClass('.isActive');
      }
    })
    return $(section);
  }

  $('.sm-docs-nav-section-text').click(function(e) {
    toggleSection(this);
    setTimeout(scrollToTop, 1);
  });

  $('.sm-docs-nav-sublink').click(function() {
    if (!$(this).hasClass('isActive')) {
      var section = $(this).parent().parent().prev();
      $('.sm-docs-nav-sublink.isActive').removeClass('isActive');
      $(this).addClass('isActive');
      if (!section.hasClass('isActive')) {
        var sublink = $(this);
        var href = sublink.attr('href');
        closeContentSection();
        openContentSection(parseHash(href));
        section.addClass('isActive');
        setTimeout(function() {
          $(document).scrollTop($('[id="'+href.replace(/^#/,'')+'"]').offset().top - 15);
        }, 100);
      }
    }
  })

  
  $('.sm-docs-section-wrap a').each(function() {

    if (!$(this).attr('href').match(/^#/)) {
      $(this).attr('target', '_blank');
    }

  })

  $('.sm-docs-section-wrap a[href^="#"]').click(function() {
    var ref = $(this).attr('href');
    var sub = ref.match(/[^\+]+\+[^\+]+/);
    if (sub) {
      getNavSub(ref).click();
      getNavSection(ref).addClass('isOpen');
    } else {
      getNavSection(ref).click();
      
    }
  })



  $('.sm-docs-nav-title').click(function() {
    var splashContent = $('.sm-documentation-content').children().first();
    if (!splashContent.hasClass('isActive')) {
      closeContentSection();
      openContentSection({section: $(this).attr('rel')})
      $('.sm-docs-nav-section-text.isActive, .sm-docs-nav-sublink.isActive').removeClass('isActive');
      window.location.hash = '';
    }
  })











})(jQuery)