// 
//  assembleIndex.js
//  ----------------
//
//  Assembles the front-end model index
//
var _ = require('./help.js');


module.exports = (function() {

	function buildIndex(data) {

    function getModelRef(str) {
      var match = str.match(/\|([^\|]+)$/);
      if (match) {
        return match[1];
      } else {
        console.error('No model reference in this string');
      }
    }

    function getModelObj(str, data) {
      var list = str.split('|');
      return data[list[0]][list[1]][list[2]];
    }

		var output = '      <ul class="sm-index-droplist sm-droplist-sections">\n';

		// SECTION LOOP
		for (var section in data) {

			if (Object.keys(data[section]).length === 0) continue;

			var sectionUrl = encodeURIComponent(section);
			output += '        <li class="sm-index-dropitem" rel="'+section+'">\n';
			output += '          <div class="sm-index-dropbtn sm-dropbtn-section">\n';
			output += '            <span class="sm-index-itemtext">'+section+'</span>\n';
			output += '            <span class="sm-index-alltext" data-url="'+sectionUrl+'">All</span>\n';
			output += '          </div>\n';
			
			if (Object.keys(data[section]).length > 0) {
				output += '          <ul class="sm-index-droplist sm-droplist-groups">\n';

				// GROUP LOOP
				for (var group in data[section]) {
					var groupUrl = sectionUrl + '|' + encodeURIComponent(group);
					output += '            <li class="sm-index-dropitem" rel="'+group+'">\n';
					output += '              <div class="sm-index-dropbtn sm-dropbtn-group">\n';
					output += '                <span class="sm-index-itemtext">'+group+'</span>\n';
					output += '                <span class="sm-index-alltext" data-url="'+groupUrl+'">All</span>\n';
					output += '              </div>\n';

					// MODEL LOOP
					if (Object.keys(data[section][group]).length > 0) {
						output += '              <ul class="sm-index-droplist sm-droplist-models">\n';

						for (var model in data[section][group]) {

              var M = data[section][group][model];
							var modelUrl = groupUrl + '|' + encodeURIComponent(model);

							output += '                <li class="sm-index-dropitem" rel="'+model+'">\n';
              if (M.scaffold) {
                output += '                  <div class="sm-index-itembtn scaffold" data-url="'+modelUrl+'">\n';
              } else {
                output += '                  <div class="sm-index-itembtn" data-url="'+modelUrl+'">\n';
              }
              output += '                    <div class="sm-index-itemicons"><span class="fa fa-code"></span>';


              if (M.scaffold) {

                output += '<span class="fa fa-info-circle"></span>';
                output += '<div class="sm-index-scaffold"><div class="sm-index-scaffold-info">';
                output += '<div class="sm-index-scaffold-heading">Scaffold Model</div>';

                if (M.note) {
                  output += '  <p class="sm-index-scaffold-note"><span>'+M.note+'</span></p>';
                }
                
                output += '  <ul class="sm-index-scaffold-list">';
                output += '    <li><span class="scaffold-list-item">File:</span><span>' + M.file + '</span></li>';
                output += '    <li><span class="scaffold-list-item">Name:</span><span>' + M.name + '</span></li>';

                if (M.author) {
                  output += '<li><span class="scaffold-list-item">Author:</span><span>' + M.author + '</span></li>';
                }
                if (M.global) {
                  output += '<li><span class="scaffold-list-item">Global:</span><span>Yes</span></li>';
                }
                if (M.js.length > 0) {
                  output += '<li><span class="scaffold-list-item">JS:</span><span>' + M.js.join(', ') + '</span></li>';
                }
                if (M.references.length > 0) {
                  output += '<li><span class="scaffold-list-item">Calls:</span>';
                  output += _.removeDuplicates(M.references.map(function(ref) {
                    return '<span class="sm-index-scaffold-link" data-url="'+ref+'">'+getModelRef(ref)+'</span>';
                  })).reverse().join(', ') + '</li>';
                }
                if (M.includes.length > 0) {
                  output += '<li><span class="scaffold-list-item">Includes:</span>';
                  output += _.removeDuplicates(M.includes.map(function(ref) {
                    if (getModelObj(ref, data).scaffold) {
                      return getModelRef(ref);
                    } else {
                      return '<span class="sm-index-scaffold-link" data-url="'+ref+'">'+getModelRef(ref)+'</span>';
                    }
                  })).reverse().join(', ') + '</li>';
                }
 

                output += '  </ul>';

                output += '</div></div>';

              } else {

                output += '<span class="fa fa-eye"></span>';

                if (M.references.length > 0) {
                  output += '<span class="fa fa-cubes"></span></div>';
                } else {
                  output += '</div>';
                }
              }

              if (M.scaffold) output += '</div>';
              
							if (M.wip) {
                output += '                    <span class="sm-index-wip">';
								output += '			        	       <i class="fa fa-exclamation-circle sm-wip-icon"></i>WIP';
                output += '                    </span>';
							}

							output += '                    <span class="sm-index-itemtext">'+model+'</span>\n';
							output += '                  </div>\n';
							output += '                </li><!-- end dropitem -->\n';
						}

						output += '              </ul><!-- end model list -->\n';

					}

					output += '            </li><!-- end group item -->\n';

				}

				output += '          </ul><!-- end group list -->\n';

			}

			output += '        </li><!-- end section item -->\n'

		}

		output += '      </ul><!-- end index -->'

		return output;

	}


	return buildIndex;


})()