// 
//  collect.js
//  ----------
//
//  Collect model data from file contents and return model object
//
var _ = require('./help.js'),
    parse = require('./parse.js'),
    gill = require('./highlight.js'),
    sass = require('node-sass'),
    prism = require('../node_modules/prismjs/components/prism-core.js');

    require('../node_modules/prismjs/components/prism-css.js');
    require('../node_modules/prismjs/components/prism-scss.js');



module.exports = (function() {

  var jsSource, jsDependencies, JS = {};

  function findModels(content) {
    return _.find(content, /(\/\*{3}[^]+?\/\*{3}\s*?end-model\s*?\*{3}\/)/g);
  }

  function getComment(content) {
    return _.find(content, /(\/\*{3}[^]+?\*{3}\/)/g, false);
  }

  function getFromComment(comment, exp) {
    return _.find(comment, exp, false);
  }

  function getSass(content) {
    return _.find(content, /\*{3}\/\s*([^]*?)\/\*{3}\s*end-model/g, false);
  }

  function addToJs(paths) {
    paths.forEach(function(path) {
      if (!JS.hasOwnProperty(path)) {
        JS[path] = '// ' + jsSource + path + '\n\n' + _.contents(jsSource + path) + '\n\n';
      }
    })
  }

  function getJsDependencies(files) {
    var globalJs = '';

    files.forEach(function(path) {
      globalJs += '// ' + jsSource + path + '\n\n' + _.contents(jsSource + path) + '\n\n';
    })
    return globalJs;
  }

  function getModelCss(name, globalCss) {
    var expr = new RegExp('\\/\\*{3}[^]+?@name:\\s*' + name + '\\s+[^]+?\\*{3}\\/\\s*([^]*?)\\/\\*{3}\\s*end-model\\s*'),
        match = globalCss.match(expr),
        output = '';
    output += match ? match[1].trim() : '???';
    return output;
  }

  function checkModelExists(obj, name, group, section) {
    var key = section + '__' + group + '__' + name;
    if (obj.hasOwnProperty(key)) {
      _.err('duplicate model: ' + name);
    }
    obj[key] = true;
    return true;
  }

  function createTemplateVersions(template) {
    var pattern = /\{#[^]*?#\}/g;
    var comments = template.match(pattern);
    var output = {
      with: null,
      without: template.replace(pattern,'').trim(),
      untouched: template
    }
    if (comments) {
      output.with = template.replace(/\{#|#\}/g,'').trim();
    }
    return output;
  }


  function getModelData(path, models, css, root) {
    
    var archive = {},
        output = models || {},
        content = _.contents(path);

    findModels(content).forEach(function(model) {

      var comment       = getComment(model),
          name          = getFromComment(comment, /@name:\s*(.+)/g),
          group         = getFromComment(comment, /@group:\s(.+)/g), 
          section       = getFromComment(comment, /@section:\s*(.+)/g);

      if (checkModelExists(archive, name, group, section)) {
        
        output[name] = {

          file:         path,
          section:      section,
          group:        group,
          display:      getFromComment(comment, /@display:\s*(.+)/g),
          name:         name,
          author:       getFromComment(comment, /@author:\s(.+)/g),
          scaffold:     getFromComment(comment, /(@scaffold)/g) === '@scaffold',
          wip:          getFromComment(comment, /(@wip)/g) === '@wip',
          note:         getFromComment(comment, /@note:\s(.+)/g) || null,
          js:           getFromComment(comment, /@js:\s(.+)/g),
          sass:         getSass(model).trim(),
          css:          getModelCss(name, css),
          includes:     getFromComment(comment, /@include:\s(.+)/g) || null,
          global:       getFromComment(comment, /(@global)/g) === '@global',
          template:     getFromComment(comment, /={3}([^]+)?\*{3}\//g).trim()

        };

        var m = output[name];
        var templateVersions = createTemplateVersions(m.template);
        
        m.template = templateVersions.without;
        m.templateUntouched = templateVersions.untouched;
        m.templateFull = templateVersions.with;

        if (m.templateFull) {
          m.parsedFull = parse(m.templateFull, root);
        }
        
        m.parsed = parse(m.template, root);

        m.js = m.js ? m.js.split(/, */) : [];
        m.includes = m.includes ? m.includes.split(',').map(_.trim) : [];
        

      }
    })

    return output;

  }


  return function(filePath, models, css, root) {
    return {
      models: getModelData(filePath, models, css, root)
    }
  }

})();