//  
//  files.js
//  --------
//
//  Get sass files of various sorts
//

var fs = require('fs');


module.exports = (function() {


  // Regular Expressions

  var SASS_FILE = /(\.scss$)/g,
      SASS_PARTIAL = /(_[\w\-]+?\.scss)$/g,
      NODE_MODULES = /(.*node_modules.*)/g;


  addSlash = function(path) {
    return (/\/$/g).test(path) ? path : path + '/';
  }


  prepend = function(paths, root) {
    return paths.map(function(e) {
      return root + e;
    });
  }


  isDir = function(path) {
    return fs.statSync(path).isDirectory();
  }


  isFile = function(path) {
    return isDir(path) ? false : true;
  }


  getPaths = function(dirPath) {
    return prepend(fs.readdirSync(dirPath), addSlash(dirPath));
  }


  getFilesDeep = function(dir, excludeFn, output) {
    var exFn = excludeFn || null,
        dirPaths = getPaths(dir),
        files = output || [],
        dirs,
        i;

    dirPaths = exFn ? dirPaths.filter(exFn) : dirPaths;
    files = files.concat(dirPaths.filter(isFile))
    dirs = dirPaths.filter(isDir);

    for (i = 0; i < dirs.length; i++) {
      files = getFilesDeep(dirs[i], exFn, files);
    }

    return exFn ? files.filter(exFn) : files;
  }

  isSassPartial = function(path) {
    return (SASS_PARTIAL).test(path);
  }


  isNotPartial = function(path) {
    return !isSassPartial(path);
  }


  notModulesIsSass = function(path) {
    if (isDir(path)) return !(/.*node_modules.*/g).test(path);
    else return (/(\.scss)$/g).test(path);
  }


  getSassFiles = function(path) {
    return getFilesDeep(path, notModulesIsSass);
  }


  getSassPartials = function(path) {
    return getSassFiles(path).filter(isSassPartial);
  }


  getSassBuilds = function(path) {
    return getSassFiles(path).filter(function(p) {
      return !(/(_[\w\-]+\.scss)$/g).test(p);
    });
  }


  return {
    getSassFiles: getSassFiles,
    getSassBuilds: getSassBuilds
  }

})();
