// Helpers

var fs = require('fs');

module.exports = (function() {

	var os = process.platform;
	var slash = os === 'win32' ? '\\' : '\/';
	var notSlash = os === 'win32' ? '\/' : '\\';

	function truePath(path) {
    var reg;
    if (os === 'win32') {
      reg = new RegExp('\\/', 'g');
    } else {
      reg = new RegExp('\\\\', 'g');
    }
		return path.replace(reg, slash);
	}

	function contents(file) {
  	return fs.readFileSync(file, 'utf8');
	}

	function err(msg) {
		console.error('\x1b[41m%s\x1b[0m', 'ERROR - ' + msg);
		process.exit(1);
	}

	function trim(str) {
		return str.trim();
	}


	function toArray(thing) {
    if (Array.isArray(thing)) return thing;
    else {
      var arr = []; arr.push(thing);
      return arr;
    }
  }

  function ifExists(directory, success, fail) {
    var pass = true;
    try {
      fs.statSync(directory);
    } catch(e) {
      pass = false;
      if (fail) fail(directory, e);
    }
    if (pass && success) success(directory);
  }

  function isDir(path) {
    return fs.statSync(path).isDirectory();
  }


	function inArray(thing, arr) {
		return arr.indexOf(thing) > -1;
	}


  function get2nd(obj) {
  	return obj['2'];
  }


  function removeDuplicates(arr) {
  	return arr.filter(function(item, i) {
      return arr.lastIndexOf(item) === i;
    })
  }


  function endSlash(path) {
  	if (path) {
  		var reg = new RegExp('\\' + slash + '$', 'g');
  		return path.replace(reg, '') + slash;
  	} else {
  		return '';
  	}
  }


  function startSlash(path) {
  	if (path) {
  		var reg = new RegExp('^\\' + slash, 'g');
  		return path.replace(reg, '') + slash;
  	} else {
  		return '';
  	}
  }


	function copy(data) {
  	return JSON.parse(JSON.stringify(data));
	}


		 // Returns random string of numbers and upper case letters of a specified length.
	function makeRandomID(num) {
    var id = '', i = 1,
    		random = function(min, max) {
    			return Math.floor(Math.random() * (max - min + 1) + min);
  			};
    while (i <= num) {
      if (i % 2 === 0) {
        id += String.fromCharCode(random(65, 90));
      } else {
        id += String.fromCharCode(random(48, 57));
      }
      i++;
    }
    return id;
  }


  // Find a series of matches from arbitrary number of regular expressions and
  // with any number of captured sequences and return an array of single value
  // content = the string to search in
  // regs = the array of regular expressions to look for
  // arr = whether you want an array back or just a single value
  // num = the number of captured sequences in the regular expressions
  function findInContent(content, regs, arr, num) {
    var matches = [], match, arrayOut = arr, captured = num ? num : 1, i, m;

    if (arrayOut === undefined) arrayOut = true;

    toArray(regs).forEach(function(rgex) {
      while ((match = (rgex).exec(content)) !== null) {
        if (captured === 1) matches.push(match[1])
        else {
          m = {};
          for (i = 1; i <= captured; i++ ) m[i] = match[i];
          matches.push(m);
        }
      }
    })

    return arrayOut ? matches : matches[0];
  }

  function find(content, regs, arr, num) {
    var matches = [], match, arrayOut = arr, captured = num ? num : 1, i, m;

    if (arrayOut === undefined) arrayOut = true;

    toArray(regs).forEach(function(rgex) {
      while ((match = (rgex).exec(content)) !== null) {
        if (captured === 1) matches.push(match[1])
        else {
          m = {};
          for (i = 1; i <= captured; i++ ) m[i] = match[i];
          matches.push(m);
        }
      }
    })

    return arrayOut ? matches : matches[0];
  }

  function prettyJson(data) {
  	return JSON.stringify(data, null, 2);
  }

  function cl(thing) {
  	console.log(thing);
  }

  function getIndexesFor(content, reg) {
    var indexes = [],
        match;

    while ((match = (reg).exec(content)) !== null) {
      indexes.push(match.index);
    }

    return indexes;
  }

  function getNested(content, start, openChar, closeChar) {

    var i = start, output = "", open = 0, closed = 0;
    var openChar = openChar || '{',
        closeChar = closeChar || '}';

    for (; (open === 0) || (open > 0 && open !== closed); i++) {
      output += content.charAt(i);
      if (content.charAt(i) === openChar) open++;
      if (content.charAt(i) === closeChar) closed++;
    }

    return output.trim();

  }

	return {

		os: os,

		truePath: truePath,

		slash: slash,

		contents: contents,

		err: err,

		trim: trim,

		toArray: toArray,

	  ifExists: ifExists,

    isDir: isDir,

		inArray: inArray,

	  get2nd: get2nd,

	  removeDuplicates: removeDuplicates,

	  endSlash: endSlash,

	  startSlash: startSlash,

  	copy: copy,

  	makeRandomID: makeRandomID,

	  findInContent: findInContent,

	  find: find,

	  prettyJson: prettyJson,

	  cl: cl,

	  getIndexesFor: getIndexesFor,

	  getNested: getNested

	}



})()
