// 
//  highlight.js
//  ------------
//
//  Custom syntaxt highlighting for built-in templating language (prism options weren't working properly)
//

var _ = require('./help.js');

module.exports = (function(){


  function highlightPunctuation(markup) {
    return markup
      .replace(/</g, '&lt;')
      .replace(/>/g, '&gt;')
      .replace(/(&lt;\/|&lt;|\/&gt;|&gt;)/g, '[^PUNC^]$1[$PUNC$]');
  }

  function highlightTags(markup) {
    return markup
      .replace(/&lt;\[\$PUNC\$\]([a-zA-Z\d]+)/g, '&lt;[$PUNC$][^TAG^]$1[$TAG$]')
      .replace(/&lt;\/\[\$PUNC\$\]([a-zA-Z\d]+)/g, '&lt;/[$PUNC$][^TAG^]$1[$TAG$]');
  }

  function highlightInnerHtml(markup) {
    return markup
      .replace(/&gt;\[\$PUNC\$\]([^]+?)\[\^PUNC\^\]&lt;/g, '&gt;[$PUNC$][^TEMP^]$1[$TEMP$][^PUNC^]&lt;')
  }

  function highlightAttributeWithValues(markup) {
    return markup
      .replace(/([a-zA-Z\-]+)="([^"]*)?"/g, '[^ATTR-NAME^]$1[$ATTR-NAME$]="[^ATTR-VALUE^]$2[$ATTR-VALUE$]"')
      .replace(/([a-zA-Z\-]+)='([^']*)?'/g, "[^ATTR-NAME^]$1[$ATTR-NAME$]='[^ATTR-VALUE^]$2[$ATTR-VALUE$]'");
  }

  function highlightAttributeAlone(markup) {

    var output = markup,
        attrsAlone,
        attrs = [],
        newAttrs;
    
    attrsAlone = markup.match(/(TAG\$\]|PUNC\$\])[\w\s\-]+?(\[\^ATTR-NAME|\[\^PUNC)/g);

    if (attrsAlone) {
      attrs = attrsAlone
        .filter(function(attr) {
          return attr
            .replace(/PUNC\$\]|\[\^PUNC|TAG\$\]|\[\^ATTR-NAME/g, '')
            .trim() != '';
        });
    }

    newAttrs = attrs.map(function(input) {
      var wrapped = input;
      var content = input
        .replace(/.+\]/,'')
        .replace(/\[.*/, '')
        .trim()
        .split(' ')
        .forEach(function(attr) {
          wrapped = wrapped.replace(attr, '[^ATTR-NAME^]' + attr + '[$ATTR-NAME$]');
        })
      return wrapped;
    });

    newAttrs.forEach(function(attr, i) {
      output = output.replace(attrs[i], newAttrs[i]);
    })

    return output;
  }

  function highlightGillAreas(markup) {
    return markup
      .replace(/(\{\{\+|\+\}\}|\{\{|\}\}|\{\+|\+\}|\[\[|\]\])/g, '[^GILL^]$1[$GILL$]');
  }

  function highlightGillTokens(str) {
    
    if (str.startsWith('`')) {
      str = str
        .replace(/`([^]*?)`/, '`[^GILL-VALUE^]$1[$GILL-VALUE$]`')
        .replace(/`/g, '[^GILL-PUNC^]`[$GILL-PUNC$]');
    }

    if (str.match(/^\s*[\w\-]+:\s*[\w\-]+\s*/)) {
      str = str
        .replace(/(\s*[\w\-]+)(:)(\s*[\w\-]+\s*)/, '[^GILL-KEY^]$1[$GILL-KEY$][^GILL-PUNC^]$2[$GILL-PUNC$][^GILL-KEY^]$3[$GILL-KEY$]')
    }

    if ((/^[\w\s\-]+$/).test(str)) {
      var reg = new RegExp('(' + str.trim() + ')');
      str = str.replace(reg, '[^GILL-STR^]$1[$GILL-STR$]');
    }

    return str;

  }

  function getGillAreas(markup) {
    var areas = markup.match(/(\[\$GILL\$\][^]+?\[\^GILL\^\])/g) || [],
        output = markup,
        content;

    areas = areas
      .filter(function(area) {
        var content = (/\[\$GILL\$\]([^]*)?\[\^GILL\^\]/g).exec(area)[1];
        return !content.match(/\[\^[^]*\^\]/g) && content.trim() != '';
      })
      .map(function(area) {
        var content = (/\[\$GILL\$\]([^]*?)\[\^GILL\^\]/g).exec(area)[1];
        var filled = area.replace(content, highlightGillTokens(content));
        output = output.replace(area, filled);
      })

    output = output
      .replace('[[[$GILL$][^GILL-STR^]', '[[[$GILL$][^GILL-KEY^]')
      .replace('[$GILL-STR$][^GILL^]]]','[$GILL-KEY$][^GILL^]]]')
      .replace(/\{\+\[\$GILL\$\]\[\^GILL-STR\^\]/g,'{+[$GILL$][^GILL-KEY^]')
      .replace(/\[\$GILL-STR\$\]\[\^GILL\^\]\+\}/g,'[$GILL-KEY$][^GILL^]+}');

    return output;
  }


  function highlightAttrSyntax(markup) {
    return markup
      .replace(/\[\$ATTR-NAME\$\]=("|')/g, '[$ATTR-NAME$][^PUNC^]=$1[$PUNC$]')
      .replace(/\[\$ATTR-VALUE\$\]("|')/g, '[$ATTR-VALUE$][^PUNC^]$1[$PUNC$]');
  }


  function highlightRevert(markup) {
    return markup
      .replace(/\[\^[\w\-]+?\^\]|\[\$[\w\-]+?\$\]/g, '')
      .replace(/&lt;/g, '<')
      .replace(/&gt;/g, '>');
  }

  function highlightConvert(markup) {
    return markup
      .replace(/\[\^TEMP\^\]|\[\$TEMP\$\]/g, '')
      .replace(/\[[\$\w\-]+\]/g, '</span>')
      .replace(/\[\^(GILL-)*PUNC\^\]/g, '<span class="token punctuation">')
      .replace(/\[\^GILL\^\]/g, '<span class="token punctuation">')
      .replace(/\[\^GILL-KEY\^\]/g, '<span class="token gill-key">')
      .replace(/\[\^GILL-STR\^\]/g, '')
      .replace(/\[\^ATTR-NAME\^\]/g, '<span class="token attr-name">')
      .replace(/\[\^ATTR-VALUE\^\]/g, '<span class="token attr-value">')
      .replace(/\[\^TAG\^\]/g, '<span class="token tag">');
  }

  /**
  * Highlights model templates with custom template language (Gill)
  *
  * punctuation = PUNC, GILL, GILL-PUNC
  * tag = TAG
  * attr-name = ATTR-NAME
  * attr-value = ATTR-VALUE, GILL-VALUE
  * gill-key = GILL-KEY
  *
  */
  function highlight(markup) {
    var output = markup || _.err('no markup passed to highlight');

    output = highlightPunctuation(output);
    output = highlightTags(output);
    output = highlightInnerHtml(output);
    output = highlightAttributeWithValues(output);
    output = highlightGillAreas(output);
    output = highlightAttrSyntax(output);
    output = getGillAreas(output);
    output = highlightAttributeAlone(output);
    output = highlightConvert(output);

    return output
  }

  highlight('<h1{+attrs+}>{{+ content: p [[content]] Insert H1 Header Content Here +}}</h1>');

  return highlight;


})();