//
//  makecss.js
//  -------
//
//  Scrape project sass files and compile a global css file
//

var _ = require('./help.js'),
    sass = require('node-sass');


module.exports = (function() {

  mergeWith = function(arr, mergewith) {
    return mergewith.concat(arr);
  }

  getImportCalls = function(path) {
    return _.find(_.contents(path), /(@import\s*(?:'|")([\w\/\-\.]+)('|");)/g);
  }

  getBuildFileDir = function(path) {
    return _.find(path, /(.+?(?:\\|\/))[\w\-\.]+?(?:\.scss)$/g, false);
  }

  prepImportValue = function(val) {
    var end = _.find(val, /((?:[^\/\\]+)$)/g, false);
    return val.replace(/([^\/\\]+)$/g, '_' + end.replace(/^_/g,'').replace(/(\.scss)$/g, '') + '.scss');
  }

  getImportValue = function(importCall) {
    var reg = /@import\s+(?:'|")(.+?)(?:'|");/g;
    return prepImportValue(_.find(importCall, reg, false));
  }

  cleanImportPath = function(importPath) {
    return importPath.replace(/(\/|\\)[\w\-\.]+\1\.\.\1/g, '/').replace(/(\/|\\)\.\1/g, '/');
  }

  getImportPaths = function(path) {
    return getImportCalls(path).map(function(imp) {
      return cleanImportPath(getBuildFileDir(path) + getImportValue(imp));
    })
  }

  fillImports = function(path, global, logged) {

    var statements = getImportCalls(path),
        content = _.contents(path),
        logged = logged || {},
        global = global || "";

    statements.forEach(function(imp) {

      var impPath = cleanImportPath(getBuildFileDir(path) + getImportValue(imp));

      if (!logged.hasOwnProperty(impPath)) {

        logged[impPath] = true;
        content = content.replace(imp, fillImports(impPath, global, logged)).trim();

      }

      else content = content.replace(imp, '').trim();

    })

    return content;

  }

  return function(files, style) {

    // var buildImports = [];
    var Global = "",
        Logged = {};

    files.forEach(function(fl) {

      Global += fillImports(fl, Global, Logged);

    });

    return sass.renderSync({
      data: Global,
      outputStyle: style
    }).css.toString();

  }


})();