//
//  sort.js
//
//  Sort sections by options.project_structure
//  Sort groups and models alphabetically  
//


var _ = require('./help.js');


module.exports = (function() {


  // If project_structure defined, return correct order
  // Else, return empty object
  function create(structure) {
      var sectionOrder = {};
      if (structure) {
        structure.forEach(function(section) {
          sectionOrder[section] = {};
        })
      }
      return sectionOrder
    }


  // Sort by display
  // Gather groups into alphabetically ordered objects
  function collect(data, structure) {

    var output = {};
    output.data = sortby(data, 'display');
    output.structure = structure;
    output.data.forEach(function(model) {
      if (!Array.isArray(output.structure[model.section])) {
        output.structure[model.section] = [];
      }
      if (output.structure[model.section].indexOf(model.group) < 0) {
        output.structure[model.section].push(model.group);
      }
    })
    for (var section in output.structure) {
      if (!Array.isArray(output.structure[section])) {
        output.structure[section] = [];
      }
      output.structure[section] = output.structure[section]
        .sort(function(a,b) {
          aV = a.toUpperCase();
          bV = b.toUpperCase();
          return (aV < bV) ? -1 : (aV > bV) ? 1 : 0;
        })
      output.structure[section] = create(output.structure[section]);
    }
    
    return assign(output.data, output.structure);
  }


  // Assign models to section/group object
  function assign(data, structure) {
    var output = _.copy(structure);
    data.forEach(function(model) {
      structure[model.section][model.group][model.display] = model;
    })
    return structure;
  }


  // Returns array sorted by specific key
  function sortby(arr, properties) {
      
      var keys = [], output;
      if (Array.isArray(properties)) {
        keys = properties;
      } else {
        keys.push(properties);
      }

      keys.forEach(function(key) {
        output = arr.sort(function(a, b) {
          var aValue = a[key].toUpperCase(),
              bValue = b[key].toUpperCase();
          return (aValue < bValue) ? -1 : (aValue > bValue) ? 1 : 0;
        })
      })

      return output;
      
    }



  return {
    create: create,
    collect: collect,
    sortby: sortby
  }


})()